/***************************************************************************
Copyright (c) 2006 Mateusz Bilski, KateOS Project
 
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
 
*Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.
 
*Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE. 
****************************************************************************/

#ifndef DOCUMENT_H
#define DOCUMENT_H

#include "stvaluelist.h"
#include "ststring.h"

/*!
	\brief StDocument represents an XML document.
	
	You can read XML an file into StDocument using StXMLParser.
*/

class StDocument
{
	public:
		class Node
		{
			public:
				enum Type { Element, Attribute, Data };
			
				Node( const StString& n, Type t, const StString& v, Node* p ) { name = n; type = t; value = v; parent = p; }
								
				StString getName() { return name; }
				Type getType() { return type; }
				StString getValue() { return value; }
				
				/*! \return pointer to parent node. */
				Node* getParent() { return parent; }
				
				void addChild( Node* child) { childNodes.push_back( child ); }
				
				bool hasChildNodes() { return !childNodes.empty(); }
				
				/*! \return a reference to the list of child nodes. */
				StValueList<Node*>& getChildNodes() { return childNodes; }
			private:
				StString name, value;
				Type type;
				Node* parent;
				StValueList<Node*> childNodes;
		};
		
		/*! Sets pointers to NULL. */
		StDocument() { currentElement = NULL; rootElement= NULL; }
		
		/*! Recursively deletes all nodes. */
		~StDocument() { clean( rootElement ); delete rootElement; }
		
		/*! \return first node. */
		Node* getRootElement() { return rootElement; }		
	private:
		void clean( Node* node );
	
		Node* rootElement;
		Node* currentElement;
		
		friend class StXMLParser;
};

#endif //DOCUMENT_H
