/***************************************************************************
Copyright (c) 2006 Peter Korzuszek, Damian Rakowski, KateOS Project
 
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
 
*Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.
 
*Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE. 
****************************************************************************/

#ifndef SMART_TOOLS_FS_H
#define SMART_TOOLS_FS_H

#include "ststring.h"
#include "stfile.h"
#include "stdir.h"

/*!
	\brief Give informations about files and manage on directiories.
	
	This class include static functions which gives informations about files and
	allows to create and remove directories.
*/

class StFs
{
	public:
		StFs();
		
		static bool fileExists( const StString& fileName, bool followSymLinks=false )
			{ StFile file( fileName, followSymLinks ); return file.exists(); }
		
		static bool isDirectory( const StString& fileName, bool followSymLinks=false )
			{ StFile file( fileName, followSymLinks ); return file.isDirectory(); }
		static bool isSymLink( const StString& fileName, bool followSymLinks=false )
			{ StFile file( fileName, followSymLinks ); return file.isSymLink(); }
		static bool isRegular( const StString& fileName, bool followSymLinks=false )
			{ StFile file( fileName, followSymLinks ); return file.isRegular(); }
		static bool isCharacterDevice( const StString& fileName, bool followSymLinks=false )
			{ StFile file( fileName, followSymLinks ); return file.isCharacterDevice(); }
		static bool isBlockDevice( const StString& fileName, bool followSymLinks=false )
			{ StFile file( fileName, followSymLinks ); return file.isBlockDevice(); }
		static bool isFifo( const StString& fileName, bool followSymLinks=false )
			{ StFile file( fileName, followSymLinks ); return file.isFifo(); }
		static bool isSocket( const StString& fileName, bool followSymLinks=false )
			{ StFile file( fileName, followSymLinks ); return file.isSocket(); }
		
		static bool isReadable( const StString& fileName, bool followSymLinks=false )
			{ StFile file( fileName, followSymLinks ); return file.getPermissions().canIRead(); }
		static bool isWriteable( const StString& fileName, bool followSymLinks=false )
			{ StFile file( fileName, followSymLinks ); return file.getPermissions().canIWrite(); }
		static bool isExecutable( const StString& fileName, bool followSymLinks=false )
			{ StFile file( fileName, followSymLinks ); return file.getPermissions().canIExecute(); }
		
		/** Domain of mkdir(). */
		static bool createDirectory( const StString& filename, bool recursively=false )
			{ return mkdir( filename, recursively ); }
		
		/** Create directory. */
		static bool mkdir( const StString& filename, bool recursively=false );
		
		/** Remove file or directory. */
		static bool remove( const StString& fileName, bool recursively=false );
		
		/** \return directory name without full path */
		static StString dirname( const StString& path );
		
		static StString realpath( const StString& path );
		
		//void setFollowSymLinks( bool opt ){ followSymLinks = opt; }
	
	private:
		//StString prefix;
		//bool followSymLinks;
		
		void init();
};

#endif //SMART_TOOLS_FS_H
